package zetabank.model;

import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.List;

public class CalcolatoreRateCostanti extends CalcolatoreRate {

	@Override
	public Finanziamento calcola(TipoFinanziamento tipo, double capitaleDaFinanziare, int durataRichiesta, Periodicita periodicita) {
		if (tipo == null || tipo.getCategoria() != SchemaFinanziamento.RATE_COSTANTI)
			throw new IllegalArgumentException("tipo");
		if (capitaleDaFinanziare <= 0)
			throw new IllegalArgumentException("capitaleDaFinanziare");
		if (durataRichiesta <= 0)
			throw new IllegalArgumentException("durataRichiesta");
		if (periodicita == null || !tipo.getPeriodicitaAmmesse().contains(periodicita))
			throw new IllegalArgumentException("periodicita");
		
		// durata gi verificata
		List<Rata> listaRate = new ArrayList<>();
		int numeroRate = durataRichiesta * periodicita.numeroRatePerAnno();
		double tassoEffettivoPercentuale = tipo.getTassoAnnuoNominale() / periodicita.numeroRatePerAnno();
		double i = tassoEffettivoPercentuale / 100;
		double rata = (1 + 1.0 / (Math.pow(1 + i, numeroRate) - 1)) * i * capitaleDaFinanziare;
		for (int k = 1; k <= numeroRate; k++)
			listaRate.add(split(rata, numeroRate, k, i));
		return new Finanziamento(tipo, durataRichiesta, periodicita, listaRate, capitaleDaFinanziare);
	}

	public static void main(String[] args) {
		Finanziamento fin = CalcolatoreRate.calcolaFinanziamento(new TipoFinanziamento("rate costanti", SchemaFinanziamento.RATE_COSTANTI,
				4.0, 10, 20, new Periodicita[] { Periodicita.MENSILE, Periodicita.BIMESTRALE }), 60000, 10, Periodicita.MENSILE);
		System.out.println(fin);
		NumberFormat formatter = NumberFormat.getCurrencyInstance();
		System.out.println("Interessi complessivi: " + formatter.format(fin.sommaTotaleInteressi()));
		// ----------dettaglio rate-----------
		int k = 0;
		for (Rata r : fin.getRate()) {
			k++;
			System.out.println("Rata " + k + ", quota capitale = " + formatter.format(r.getQuotaCapitale()) + ", quota interessi = "
					+ formatter.format(r.getQuotaInteressi()));
		}
	}

}
