package zetabank.model;

import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.List;

public class CalcolatoreRateDecrescenti extends CalcolatoreRate {

	@Override
	public Finanziamento calcola(TipoFinanziamento tipo, double capitaleDaFinanziare, int durataRichiesta, Periodicita periodicita){
		if (tipo == null|| tipo.getCategoria() != SchemaFinanziamento.RATE_DECRESCENTI)
			throw new IllegalArgumentException("tipo");
		if (capitaleDaFinanziare <= 0)
			throw new IllegalArgumentException("capitaleDaFinanziare");
		if (durataRichiesta <= 0)
			throw new IllegalArgumentException("durataRichiesta");
		if (periodicita == null)
			throw new IllegalArgumentException("periodicita");
		
		//durata gi verificata
		int numeroRate = durataRichiesta * periodicita.numeroRatePerAnno();
		double quotaCapitale =  capitaleDaFinanziare / numeroRate; // costante in tutte le rate
		double tassoEffettivoPercentuale = tipo.getTassoAnnuoNominale()/periodicita.numeroRatePerAnno();
		double i = tassoEffettivoPercentuale/100;
		List<Rata> listaRate = new ArrayList<>();
		for (int k=1; k<=numeroRate; k++) {
			double quotaInteressi = i * (capitaleDaFinanziare - (k-1)*quotaCapitale);
			listaRate.add( new Rata(k, quotaCapitale, quotaInteressi) );
		}
		return new Finanziamento(tipo, durataRichiesta, periodicita, listaRate, capitaleDaFinanziare);
	}

	public static void main(String[] args){
		Finanziamento fin = CalcolatoreRate.calcolaFinanziamento(
				new TipoFinanziamento("rate decrescenti", SchemaFinanziamento.RATE_DECRESCENTI,
						4.0, 10, 20, new Periodicita[]{Periodicita.MENSILE, Periodicita.BIMESTRALE}),
				60000, 10, Periodicita.MENSILE);
		System.out.println(fin);
		NumberFormat formatter = NumberFormat.getCurrencyInstance();
		System.out.println("Interessi complessivi: " + formatter.format(fin.sommaTotaleInteressi()));
		//----------dettaglio rate-----------
		int k=0;
		for(Rata r: fin.getRate()){
			k++;
			System.out.println("Rata " + k + ", quota capitale = " + formatter.format(r.getQuotaCapitale()) 
					+ ", quota interessi = " + formatter.format(r.getQuotaInteressi())); 
		}
	}
}
