package zetabank.persistence;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.Reader;
import java.util.ArrayList;
import java.util.List;
import java.util.NoSuchElementException;
import java.util.StringTokenizer;

import zetabank.model.*;

public class MyFinReader implements FinReader {

	private List<TipoFinanziamento> tipiFinanziamento;
	private BufferedReader reader;

	public MyFinReader(Reader reader) {
		if (reader == null)
			throw new IllegalArgumentException("reader");
		
		this.reader = new BufferedReader(reader);
		tipiFinanziamento = new ArrayList<>();
	}

	public List<TipoFinanziamento> readElements() throws BadFileFormatException {

		String line = null;
		try {
			while ((line = reader.readLine()) != null) {
				StringTokenizer tokenizer = new StringTokenizer(line, "\t\n\r");
				String nome = tokenizer.nextToken().trim();
				String categoria = tokenizer.nextToken().trim();
				String tasso = tokenizer.nextToken().trim();
				String durata1 = tokenizer.nextToken().trim();
				String durata2 = tokenizer.nextToken().trim();
				String listaPeriodicitaAmmesse = tokenizer.nextToken().trim();
				// verifiche consistenza formato
				SchemaFinanziamento schema = SchemaFinanziamento.valueOf(categoria.replace(" ", "_"));
				if (!tasso.endsWith("%")) throw new IllegalArgumentException("tasso non  una percentuale");
				double tassoNominale = Double.parseDouble(tasso.substring(0, tasso.length()-1));
				int dMin = Integer.parseInt(durata1);
				int dMax = Integer.parseInt(durata2);
				if (dMax<dMin) throw new IllegalArgumentException("durata massima inferiore alla minima");
				// parsing lista periodicit
				String[] periodicitAmmesse = listaPeriodicitaAmmesse.split(",");
				List<Periodicita> periodicit = new ArrayList<>();
				for (String s: periodicitAmmesse) {
					periodicit.add(Periodicita.valueOf(s.trim()));
				}
				tipiFinanziamento.add(new TipoFinanziamento(nome, schema, tassoNominale, dMin, dMax, periodicit));
			}
		} catch (NoSuchElementException | IOException | IllegalArgumentException e) {
			throw new BadFileFormatException(e);
		}
		return tipiFinanziamento;
	}

	public static void main(String[] args) throws FileNotFoundException, BadFileFormatException{
		FinReader reader = new MyFinReader(new BufferedReader(new FileReader("Finanziamenti.txt")));
		List<TipoFinanziamento> elencoFinanziamenti = reader.readElements();
		for (TipoFinanziamento t : elencoFinanziamenti) System.out.println(t);
	}
	
}
