package zetabank.ui;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.List;
import java.util.Optional;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.WindowConstants;

import zetabank.model.Finanziamento;
import zetabank.model.Periodicita;
import zetabank.model.TipoFinanziamento;
import zetabank.ui.controller.Controller;

public class MainFrame extends JFrame implements ActionListener {

	private static final long serialVersionUID = 1L;

	private Controller controller;
	private JPanel mainPanel;
	private FinancingPane financingTable;
	private JComboBox<TipoFinanziamento> tipiFinanziamentiCombo;
	private JComboBox<Periodicita> periodicitCombo;
	private JButton calcola;
	private JTextField importo, durata, rataMassima;

	public MainFrame(Controller controller) {
		this.controller = controller;
		initGUI();
		setSize(550, 340);
	}

	private void initGUI() {

		mainPanel = new JPanel();
		getContentPane().add(mainPanel);

		setTitle("ZetaBank");
		setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);

		JLabel label = new JLabel("Simulazione Finanziamento");
		mainPanel.add(label, BorderLayout.NORTH);

		tipiFinanziamentiCombo = new JComboBox<TipoFinanziamento>(controller.getElencoTipiFinanziamento().toArray(new TipoFinanziamento[1]));
		mainPanel.add(tipiFinanziamentiCombo, BorderLayout.NORTH);

		importo = new JTextField(4);
		importo.setText("60000");
		mainPanel.add(new JLabel("Importo "), BorderLayout.NORTH);
		mainPanel.add(importo, BorderLayout.NORTH);

		durata = new JTextField(4);
		durata.setText("10");
		mainPanel.add(new JLabel("Durata anni"), BorderLayout.NORTH);
		mainPanel.add(durata, BorderLayout.NORTH);

		rataMassima = new JTextField(4);
		rataMassima.setText("700.00");
		mainPanel.add(new JLabel("Rata max "), BorderLayout.NORTH);
		mainPanel.add(rataMassima, BorderLayout.NORTH);

		Box box = new Box(BoxLayout.LINE_AXIS);
		periodicitCombo = new JComboBox<Periodicita>(Periodicita.values());
		box.add(new JLabel("Periodicit "));
		box.add(periodicitCombo);
		box.add(new JLabel("        "));

		calcola = new JButton("Calcola");
		calcola.addActionListener(this);
		box.add(calcola);
		mainPanel.add(box, BorderLayout.NORTH);

		financingTable = new FinancingPane(); // inizialmente tutta vuota
		mainPanel.add(financingTable, BorderLayout.CENTER);
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		financingTable.reset();
		TipoFinanziamento t = (TipoFinanziamento) tipiFinanziamentiCombo.getSelectedItem();
		// Finanziamento fin = controller.calcAndCheck(t, 60000, 10,
		// Periodicita.MENSILE, 700);
		try {
			double capitale = Double.parseDouble(importo.getText());
			double rataMax = rataMassima.getText().equals("") ? Double.MAX_VALUE : Double.parseDouble(rataMassima.getText());
			int durataInAnni = Integer.parseInt(durata.getText());
			Periodicita p = (Periodicita) periodicitCombo.getSelectedItem();
			// verifiche
			List<Periodicita> ammesse = t.getPeriodicitaAmmesse();
			if (!ammesse.contains(p)) {
				controller.getUserInteractor().showMessage("Periodicit non ammessa per questo tipo di finanziamento");
				return;
			}
			// esecuzione calcolo
			Optional<Finanziamento> fin = controller.calcAndCheck(t, capitale, durataInAnni, p, rataMax);
			if (fin.isPresent()) {
				financingTable.update(fin.get().getRate());
			}
		} catch (IllegalArgumentException ex) {
			controller.getUserInteractor().showMessage("Capitale, rata max e anni devono essere numeri");
			return;
		}
	}
}
